recursive subroutine draw_tree(root, text, file, circle_node) 
  ! Purpopse: 
  !    Draws a PS-tricks version of the suffix tree.
  ! Copyright (C) 2004 Bernhard Haubold
  ! Software distributed under the terms of the GNU GENERAL PUBLIC LICENSE
  ! Record of revisions:
  !    Date            Programmer           Description of change
  !    ====            ==========           =====================
  !  Jan. 17, 2004     B. Haubold           Original code
  !  May  27, 2009     B. Haubold           Fixed problem with uninitialized node pointers
  !                                         Fixed utilization of pstricks packages
  !  July 05, 2013     B. Haubold           Replaced dotnode by cnode to avoid inclusion of type 3 fonts
  !
  use stree
  use sequence_input
  implicit none
  type(node), pointer                   :: root
  type(text_row), dimension(:), pointer :: text        ! Text of the suffix tree
  character(256), intent(in)            :: file        ! Name of PsTricks file
  logical, intent(in)                   :: circle_node ! Defines the node type !circle means cnode
  character(256)                        :: filename    ! Name of pspicture
  character(256)                        :: figure      ! Name of wrapping LaTeX-file
  type(draw_node), pointer              :: draw_root
  real                                :: xpos = 0.0
  real                                :: tree_depth
  real  , parameter                   :: level_scale = 2.5
  integer   , parameter               :: unit=2
  integer    i, j
  integer                             :: max_len = 0 ! Maxmum length of input sequences
  integer                             :: max_label_length = 0
  real                                :: yoffset
  integer    text_size, row_size, label_size

  ! Adjust file names for TeX usage
  figure = trim(file)//'_fig.tex'
  filename = trim(file)//'.tex'
  allocate(draw_root)
  call copy_node(root, draw_root)
  ! Copy the tree and set x-coordinates
  call copy_tree(root, draw_root)
  call find_max_label_length(root)
  ! Set y-coordinates
  tree_depth = 0.0
  draw_root%y=0
  call set_y(draw_root)
  tree_depth = tree_depth * (-1)
  ! draw_root%y = draw_root%y*(level_scale+1.0)
  ! Write wrapper file

  ! Find length of longest line
  text_size=size(text)
  do i=1,text_size
     if(max_len < size(text(i)%r)) max_len = size(text(i)%r)
  end do
  open(unit, file=figure)
  write(unit,'(2a)') char(92), 'documentclass{article}'
  write(unit,'(2a)') char(92), 'usepackage{times}'
  write(2,'(2a)') char(92), 'usepackage{pstricks,pst-node,pst-tree,pst-text}'
  write(unit,'(2a)') char(92), 'usepackage{graphics}'
  write(unit,'(2a)') char(92), 'oddsidemargin=0cm'
  write(unit,'(2a)') char(92), 'evensidemargin=0cm'
  write(unit,'(2a)') char(92), 'pagestyle{empty}'
  write(unit,'(2a)') char(92), 'textwidth=16cm'
  write(unit,'(2a)') char(92), 'textheight=22cm'
  write(unit,'(2a)') char(92), 'begin{document}'
  write(unit,'(2a)') char(92), 'begin{center}'
  write(unit,'(2a)', advance='no') char(92), 'begin{tabular}{|c||'
  do i=2, max_len+1
     write(unit,'(a)', advance='no') 'c|'
  end do
  write(unit,'(3a)') '}',char(92),'hline'
  do i=1, max_len
     write(unit,'(a,i3)', advance='no') '&', i
  end do
  write(unit, '(6a)') char(92),char(92),char(92), 'hline',char(92),'hline'
  do i=1, text_size
     if(text_size.gt.1)then
        write(unit, '(a,i3,2a)', advance='no')  '$T_{', i, '}$ & ', text(i)%r(1)
     else
        write(unit, '(2a)', advance='no')  '$T$ & ', text(i)%r(1)
     endif
     row_size=size(text(i)%r)
     do j=2, row_size
        if(text(i)%r(j).ne.'$')then
           write(unit,'(2a)', advance='no') '&', text(i)%r(j)
        else
           write(unit,'(3a)', advance='no') '&', char(92), '$'
        end if
     end do
     row_size=row_size+1
     do j=row_size, max_len
        write(unit, '(a)', advance='no') ' & '
     end do
     write(unit, '(4a)') char(92), char(92), char(92), 'hline'
  end do
  write(unit, '(4a)') char(92), 'end{tabular}',char(92),'end{center}'
  write(unit,'(14a)') char(92), 'begin{center}', char(92), 'resizebox{', &
       char(92), 'textwidth}{!}{', char(92), 'fbox{', char(92), 'input{', trim(file),'}}}', &
       char(92), 'end{center}'
  write(unit,'(2a)') char(92), 'end{document}'
  close(unit)
  ! Write pspicture file
  yoffset = real(size(text))*(-0.7)
  open(unit, file=filename)
  write(unit,'(2a,3(f6.3,a))') char(92), 'begin{pspicture}(-1,', yoffset,')(', xpos, ', ', tree_depth, ')'
  write(unit,'(2a)') char(92), 'psset{linecolor=lightgray}'
  call draw(unit, draw_root)
  write(unit,'(2a)') char(92), 'end{pspicture}'
  write(6,'(2a)') '# PS-picture file written to ', trim(filename)
  write(6,'(2a)') '# LaTeX wrapper file written to ', trim(figure)
  close(unit)



contains 

  recursive subroutine draw(unit, draw_root)
    implicit none
    integer   , intent(in)      :: unit
    type(draw_node), pointer      :: draw_root
    
    ! Draw node
    call draw_single_node(unit, draw_root)
    ! Hop down tree
    if(associated(draw_root%child)) call draw(unit, draw_root%child)
    ! Hop along list of children
    if(associated(draw_root%sibling)) call draw(unit, draw_root%sibling)
  end subroutine draw
    

  subroutine draw_single_node(unit, draw_root)
    implicit none
    integer   , intent(in)    :: unit
    type(draw_node), pointer    :: draw_root
    type(draw_node), pointer    :: probe
    type(leaf_label), pointer   :: ll
    integer                   :: i

    if(circle_node)then
     write(unit,'(a,2(a,f7.3),2a,2(a,i3))') char(92), 'rput(',   &
             draw_root%x,                             &
             ',',                                     &
             draw_root%y+tree_depth,                  &
             '){', char(92), 'circlenode',            &
             draw_root%id,                            &
             '}',                                     &
             draw_root%id,'}}'
     else
        write(unit,'(a,2(a,f7.3),a,i3,a)') char(92), 'cnode(', &
             draw_root%x,                             &
             ',',                                     &
             draw_root%y+tree_depth,                  &
             '){1pt}{',                               &
             draw_root%id,                            &
             '}'
     endif
     if(associated(draw_root%child)) then
        probe=>draw_root%child
        ! Go through all children
        do
           if(.not.associated(probe)) exit
           write(unit,'(3a,4(a,f7.3),3a)') char(92), 'pstextpath[c]{', char(92), 'psline(', &
                draw_root%x,                                      &
                ',',                                              &
                draw_root%y+tree_depth,                           &
                ')(',                                             &
                probe%x,                                          &
                ',',                                              &
                probe%y+tree_depth,                               &
                ')}{', char(92), 'texttt{%'
           ! Write edge label
           label_size=size(probe%label)
           do i=1,label_size
              ! Check for $-character, which is a metacharacter in LaTeX
              if(probe%label(i).ne.'$') then
                 write(unit,'(a)', advance='no') probe%label(i)
              else
                 write(unit,'(2a)', advance='no') char(92), '$'
              end if
           end do
           probe=>probe%sibling
           write(unit,*) '}}'
        end do
     else
        write(unit,*) char(92), 'nput{-90}{',draw_root%id,'}{'
        ! Distinguish between single string in tree and more than one string in tree
        if(size(text).eq.1) then
           write(unit,*) draw_root%leaf%pos
        else
           ll=>draw_root%leaf
           write(unit,*) char(92), 'begin{tabular}{c}'
           do
              if(.not.associated(ll)) exit
              write(unit,*) ll%id,',',ll%pos,char(92),char(92)
              ll=>ll%next
           end do
           write(unit,*) char(92), 'end{tabular}'
        end if
        write(unit,*) '}'
     end if
  end subroutine draw_single_node


  ! Breadth-first traveral of tree to set y-values
  recursive subroutine set_y(draw_root)
    implicit none
    type(draw_node), pointer :: draw_root
    if(associated(draw_root%parent)) draw_root%y = draw_root%parent%y-(1.0*real(max_label_length)/4.0)
    if(tree_depth.gt.draw_root%y) tree_depth = draw_root%y
    if(associated(draw_root%sibling)) call set_y(draw_root%sibling)
    if(associated(draw_root%child)) call set_y(draw_root%child)
 end subroutine set_y


  ! Tree traversal, looking at root node first
  recursive subroutine copy_tree(root, draw_root)
    implicit none
    type(node), pointer      :: root
    type(draw_node), pointer :: draw_root
    type(draw_node), pointer :: dn, probe
    real                   :: x1, x2
    ! Reproduce child nodes
    if(associated(root%child)) then
       if(.not.associated(draw_root%child)) then
          allocate(dn)
          call copy_node(root%child, dn)
          dn%parent=>draw_root
          draw_root%child=>dn
       end if
    end if
    ! Reproduce sibling nodes
    if(associated(root%sibling)) then
       if(.not.associated(draw_root%sibling)) then
          allocate(dn)
          call copy_node(root%sibling, dn)
          draw_root%sibling=>dn
          dn%parent=>draw_root%parent
       end if
    end if
    if(associated(root%child)) then
       call copy_tree(root%child, draw_root%child)
    end if
    ! Take care of x-label
    if(.not.associated(draw_root%child)) then
       draw_root%x = xpos
       xpos = xpos +1
    else
       probe=>draw_root%child
       ! Get x position of first child
       x1 = probe%x
       ! Get x position of last child
       do
          if(.not.associated(probe)) exit
          x2 = probe%x
          probe=>probe%sibling
       end do
       ! Place parent in the middle of its children
       draw_root%x = x1 + (x2-x1)/2.0
    end if
    if(associated(root%sibling)) then
       call copy_tree(root%sibling, draw_root%sibling)
    end if
  end subroutine copy_tree

  subroutine copy_node(n, dn)
    implicit none
    type(node), pointer      :: n
    type(draw_node), pointer :: dn

    if(.not.associated(dn)) allocate(dn)
    dn%leaf=>n%leaf
    dn%label=>n%label
    dn%depth=n%depth
    dn%id=n%id
    nullify(dn%parent,dn%child,dn%sibling,dn%link)
  end subroutine copy_node

  recursive subroutine find_max_label_length(root)
    type(node), pointer :: root
    if(associated(root%child)) call find_max_label_length(root%child)
    if(max_label_length < size(root%label)) max_label_length = size(root%label)
    if(associated(root%sibling)) call find_max_label_length(root%sibling)
  end subroutine find_max_label_length

end subroutine draw_tree
